#ifndef _GKRLEG_H
#define _GKRLEG_H

/**
 * \file GkrLeg.h
 * \brief Drive one leg of the Geckrobot thanks to the HerkuleXLib
 * \author Benoit Puel (contact@benoitpuel.com)
 * \version 1.0
 * \date 26 December 2015
 * \copyright This project is released under the GNU GPL V3.
 */

#include <HkxGroupPosControl.h>


/**
 * \class GkrLeg
 * \brief Class to manage one leg of the Geckrobot
 * \details This class manages one leg of the Geckrobot. 
 */
class GkrLeg{
private:
  /** Group of three servos of the leg  */
  HkxGroupPosControl& _servos; 
  
  /** Class name to print the messages */
  const String _className;

  /** Communication to print messages  */
  HkxPrint& _print; 

  /** Current X,Y,Z position in 10^-1 mm  */
  int16_t _currentSpacePosition[3]; 

  /** Current theta0, theta1, theta2 position in 10^-1 degrees  */
  int16_t _currentJointPosition[3];

  /** X,Y,Z position of the shoulder in 10^-1 mm  */
  int16_t _shoulderPosition[3];

  /** center-to-center distance respectivly shoulder-elbow, elbow-wrist and wrist-driving point in 10^-1 mm  */
  int16_t _legDistances[3];

  bool _pbDetected;

  /**
   * \brief Print error
   * \details Print an error message if setup.
   * \param[in] message : content to print as an error.  
   */
  void errorPrint(const String& message){if(_print._errorMessages){_print.errorPrint(_className + message);}};

  /**
   * \brief Print warning
   * \details Print a warning message if setup.
   * \param[in] message : content to print as a warning.  
   */
  void warningPrint(const String& message){if(_print._errorMessages){_print.warningPrint(_className + message);}};

  /**
   * \brief Print info
   * \details Print an info message if setup.
   * \param[in] message : content to print as an info.  
   */
  void infoPrint(const String& message){if(_print._errorMessages){_print.infoPrint(_className + message);}};


  bool spaceToJointConversion(const int16_t spacePosition[], int16_t jointPosition[]);

  void jointToSpaceConversion(const int16_t jointPosition[], int16_t spacePosition[]);
  
  void getMaxTorque(const int16_t jointPosition[], const int16_t forces[], int16_t maxTorque[]);
  
  uint8_t applyMaxTorque(const int16_t maxTorque[], const int16_t velocity[]);


  int16_t radToDeg(float rad){return 572.96*rad;} //float rad to int 10^-1 degree
  float degToRad(int16_t deg){return (float)deg/572.96;} //int 10^-1 degree to float rad
  
  
  uint8_t updateCurrentPosition();
  
public:

  /**
   * \brief Constructor
   * \details The constructor of GkrLeg.
   * \param[in] servos : Group of 3 servos to drive. Should be stored in the following order: 0=shoulder, 1=elbow, 2=wrist 
   * \param[in] print : Communication to print messages.
   * \param[in] shoulderPosition : X, Y and Z positions of the shoulder axis in 10^-1 mm.
   * \param[in] legDistances : Center-to-center distance respectivly shoulder-elbow, elbow-wrist and wrist-driving point in 10^-1 mm.
   */
  GkrLeg(HkxGroupPosControl& servos, HkxPrint& print, int16_t shoulderPosition[], int16_t legDistances[]);

  uint8_t moveSpace(int16_t spacePosition[], uint32_t playTime, HkxMaybe<hkxLEDControl> tabLedNoValue[], int16_t force[]);

  uint8_t moveJoints(int16_t jointPosition[], uint32_t playTime, HkxMaybe<hkxLEDControl> tabLedNoValue[]);

  uint8_t getCurrentSpacePosition(int16_t spacePosition[], bool update);

  uint8_t getCurrentJointPosition(int16_t jointPosition[], bool update);

};

#endif    // _GKRLEG_H
